import torch
import torch.optim as optim
import torch.nn as nn

from EncapsulatedFunctionsDistributed import EncapsulatedFunctionsDistributed
from Model_pt import Imagenet_Red
import time
from ds_tensor import from_pt_tensor
from pycompss.api.api import compss_wait_on, compss_barrier
import sys, csv
import numpy as np
import os

def main():
    gpus = int(sys.argv[1])
    cache = sys.argv[2]
    pc = sys.argv[3]

    torch.set_default_dtype(torch.float64)
    model = Imagenet_Red(3, 1000)
    encaps_function = EncapsulatedFunctionsDistributed(num_workers=gpus)
    criterion = nn.CrossEntropyLoss
    optimizer = optim.SGD
    optimizer_parameters = {"lr": 0.01}
    x_train = torch.load("/gpfs/projects/bsc19/bsc19756/Results_EDDL_VGG/Pytorch/x_train.pt")[:3000]
    y_train = torch.load("/gpfs/projects/bsc19/bsc19756/Results_EDDL_VGG/Pytorch/y_train.pt")[:3000]
    x_train_tensor = from_pt_tensor(x_train, shape=(1, 1))
    y_train_tensor = from_pt_tensor(y_train, shape=(1, 1))
    encaps_function.build(model, optimizer, criterion, optimizer_parameters, num_gpu=gpus, num_nodes=gpus)

    jobid = os.environ['SLURM_JOB_ID']

    print(f'MATMUL GPUS: {gpus} CACHE: {cache} JOBID: {jobid} PC: {pc}')

    times_epoch = encaps_function.fit_synchronous_with_GPU(x_train_tensor, y_train_tensor, 150, 5)

if __name__ == '__main__':
    main()
